{-# LANGUAGE TypeApplications #-}


-- | Copyright  : Will Thompson and Iñaki García Etxebarria
-- License    : LGPL-2.1
-- Maintainer : Iñaki García Etxebarria
-- 
-- A render node that copies the current state of the rendering canvas
-- so a t'GI.Gsk.Objects.PasteNode.PasteNode' can draw it.
-- 
-- /Since: 4.22/

#if (MIN_VERSION_haskell_gi_overloading(1,0,0) && !defined(__HADDOCK_VERSION__))
#define ENABLE_OVERLOADING
#endif

module GI.Gsk.Objects.CopyNode
    ( 

-- * Exported types
    CopyNode(..)                            ,
    IsCopyNode                              ,
    toCopyNode                              ,


 -- * Methods
-- | 
-- 
--  === __Click to display all available methods, including inherited ones__
-- ==== Methods
-- [draw]("GI.Gsk.Objects.RenderNode#g:method:draw"), [ref]("GI.Gsk.Objects.RenderNode#g:method:ref"), [serialize]("GI.Gsk.Objects.RenderNode#g:method:serialize"), [unref]("GI.Gsk.Objects.RenderNode#g:method:unref"), [writeToFile]("GI.Gsk.Objects.RenderNode#g:method:writeToFile").
-- 
-- ==== Getters
-- [getBounds]("GI.Gsk.Objects.RenderNode#g:method:getBounds"), [getChild]("GI.Gsk.Objects.CopyNode#g:method:getChild"), [getChildren]("GI.Gsk.Objects.RenderNode#g:method:getChildren"), [getNodeType]("GI.Gsk.Objects.RenderNode#g:method:getNodeType"), [getOpaqueRect]("GI.Gsk.Objects.RenderNode#g:method:getOpaqueRect").
-- 
-- ==== Setters
-- /None/.

#if defined(ENABLE_OVERLOADING)
    ResolveCopyNodeMethod                   ,
#endif

-- ** getChild #method:getChild#

#if defined(ENABLE_OVERLOADING)
    CopyNodeGetChildMethodInfo              ,
#endif
    copyNodeGetChild                        ,


-- ** new #method:new#

    copyNodeNew                             ,




    ) where

import Data.GI.Base.ShortPrelude
import qualified Data.GI.Base.ShortPrelude as SP
import qualified Data.GI.Base.Overloading as O
import qualified Prelude as P

import qualified Data.GI.Base.Attributes as GI.Attributes
import qualified Data.GI.Base.BasicTypes as B.Types
import qualified Data.GI.Base.ManagedPtr as B.ManagedPtr
import qualified Data.GI.Base.GArray as B.GArray
import qualified Data.GI.Base.GClosure as B.GClosure
import qualified Data.GI.Base.GError as B.GError
import qualified Data.GI.Base.GHashTable as B.GHT
import qualified Data.GI.Base.GVariant as B.GVariant
import qualified Data.GI.Base.GValue as B.GValue
import qualified Data.GI.Base.GParamSpec as B.GParamSpec
import qualified Data.GI.Base.CallStack as B.CallStack
import qualified Data.GI.Base.Properties as B.Properties
import qualified Data.GI.Base.Signals as B.Signals
import qualified Control.Monad.IO.Class as MIO
import qualified Data.Coerce as Coerce
import qualified Data.Text as T
import qualified Data.Kind as DK
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map
import qualified Foreign.Ptr as FP
import qualified GHC.OverloadedLabels as OL
import qualified GHC.Records as R
import qualified Data.Word as DW
import qualified Data.Int as DI
import qualified System.Posix.Types as SPT
import qualified Foreign.C.Types as FCT

-- Workaround for https://gitlab.haskell.org/ghc/ghc/-/issues/23392
#if MIN_VERSION_base(4,18,0)
import qualified GI.Cairo.Structs.Context as Cairo.Context
import qualified GI.GLib.Structs.Bytes as GLib.Bytes
import qualified GI.Graphene.Structs.Rect as Graphene.Rect
import qualified GI.Gsk.Callbacks as Gsk.Callbacks
import {-# SOURCE #-} qualified GI.Gsk.Enums as Gsk.Enums
import {-# SOURCE #-} qualified GI.Gsk.Objects.RenderNode as Gsk.RenderNode

#else
import {-# SOURCE #-} qualified GI.Gsk.Objects.RenderNode as Gsk.RenderNode

#endif

-- | Memory-managed wrapper type.
newtype CopyNode = CopyNode (SP.ManagedPtr CopyNode)
    deriving (Eq)

instance SP.ManagedPtrNewtype CopyNode where
    toManagedPtr (CopyNode p) = p

foreign import ccall "gsk_copy_node_get_type"
    c_gsk_copy_node_get_type :: IO B.Types.GType

instance B.Types.TypedObject CopyNode where
    glibType = c_gsk_copy_node_get_type

-- | Type class for types which can be safely cast to t'CopyNode', for instance with `toCopyNode`.
class (SP.BoxedPtr o, SP.TypedObject o, O.IsDescendantOf CopyNode o) => IsCopyNode o
instance (SP.BoxedPtr o, SP.TypedObject o, O.IsDescendantOf CopyNode o) => IsCopyNode o

instance O.HasParentTypes CopyNode
type instance O.ParentTypes CopyNode = '[Gsk.RenderNode.RenderNode]

-- | Cast to t'CopyNode', for types for which this is known to be safe. For general casts, use 'Data.GI.Base.ManagedPtr.castTo'.
toCopyNode :: (MIO.MonadIO m, IsCopyNode o) => o -> m CopyNode
toCopyNode = MIO.liftIO . B.ManagedPtr.unsafeCastTo CopyNode

--- XXX Missing getter and/or setter, so no GValue instance could be generated.
#if defined(ENABLE_OVERLOADING)
type family ResolveCopyNodeMethod (t :: Symbol) (o :: DK.Type) :: DK.Type where
    ResolveCopyNodeMethod "draw" o = Gsk.RenderNode.RenderNodeDrawMethodInfo
    ResolveCopyNodeMethod "ref" o = Gsk.RenderNode.RenderNodeRefMethodInfo
    ResolveCopyNodeMethod "serialize" o = Gsk.RenderNode.RenderNodeSerializeMethodInfo
    ResolveCopyNodeMethod "unref" o = Gsk.RenderNode.RenderNodeUnrefMethodInfo
    ResolveCopyNodeMethod "writeToFile" o = Gsk.RenderNode.RenderNodeWriteToFileMethodInfo
    ResolveCopyNodeMethod "getBounds" o = Gsk.RenderNode.RenderNodeGetBoundsMethodInfo
    ResolveCopyNodeMethod "getChild" o = CopyNodeGetChildMethodInfo
    ResolveCopyNodeMethod "getChildren" o = Gsk.RenderNode.RenderNodeGetChildrenMethodInfo
    ResolveCopyNodeMethod "getNodeType" o = Gsk.RenderNode.RenderNodeGetNodeTypeMethodInfo
    ResolveCopyNodeMethod "getOpaqueRect" o = Gsk.RenderNode.RenderNodeGetOpaqueRectMethodInfo
    ResolveCopyNodeMethod l o = O.MethodResolutionFailed l o

instance (info ~ ResolveCopyNodeMethod t CopyNode, O.OverloadedMethod info CopyNode p) => OL.IsLabel t (CopyNode -> p) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.overloadedMethod @info
#else
    fromLabel _ = O.overloadedMethod @info
#endif

#if MIN_VERSION_base(4,13,0)
instance (info ~ ResolveCopyNodeMethod t CopyNode, O.OverloadedMethod info CopyNode p, R.HasField t CopyNode p) => R.HasField t CopyNode p where
    getField = O.overloadedMethod @info

#endif

instance (info ~ ResolveCopyNodeMethod t CopyNode, O.OverloadedMethodInfo info CopyNode) => OL.IsLabel t (O.MethodProxy info CopyNode) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.MethodProxy
#else
    fromLabel _ = O.MethodProxy
#endif

#endif

-- XXX Wrapping a foreign struct/union with no known destructor or size, leak?
instance BoxedPtr CopyNode where
    boxedPtrCopy = return
    boxedPtrFree = \_x -> return ()


-- method CopyNode::new
-- method type : Constructor
-- Args: [ Arg
--           { argCName = "child"
--           , argType =
--               TInterface Name { namespace = "Gsk" , name = "RenderNode" }
--           , argCType = Just "GskRenderNode*"
--           , direction = DirectionIn
--           , mayBeNull = False
--           , argDoc =
--               Documentation
--                 { rawDocText = Just "The child" , sinceVersion = Nothing }
--           , argScope = ScopeTypeInvalid
--           , argClosure = -1
--           , argDestroy = -1
--           , argCallerAllocates = False
--           , argCallbackUserData = False
--           , transfer = TransferNothing
--           }
--       ]
-- Lengths: []
-- returnType: Just (TInterface Name { namespace = "Gsk" , name = "CopyNode" })
-- throws : False
-- Skip return : False

foreign import ccall "gsk_copy_node_new" gsk_copy_node_new :: 
    Ptr Gsk.RenderNode.RenderNode ->        -- child : TInterface (Name {namespace = "Gsk", name = "RenderNode"})
    IO (Ptr CopyNode)

-- | Creates a @GskRenderNode@ that copies the current rendering
-- canvas for playback by paste nodes that are part of the child.
-- 
-- /Since: 4.22/
copyNodeNew ::
    (B.CallStack.HasCallStack, MonadIO m, Gsk.RenderNode.IsRenderNode a) =>
    a
    -- ^ /@child@/: The child
    -> m CopyNode
    -- ^ __Returns:__ A new @GskRenderNode@
copyNodeNew child = liftIO $ do
    child' <- unsafeManagedPtrCastPtr child
    result <- gsk_copy_node_new child'
    checkUnexpectedReturnNULL "copyNodeNew" result
    result' <- (wrapPtr CopyNode) result
    touchManagedPtr child
    return result'

#if defined(ENABLE_OVERLOADING)
#endif

-- method CopyNode::get_child
-- method type : OrdinaryMethod
-- Args: [ Arg
--           { argCName = "node"
--           , argType =
--               TInterface Name { namespace = "Gsk" , name = "CopyNode" }
--           , argCType = Just "const GskRenderNode*"
--           , direction = DirectionIn
--           , mayBeNull = False
--           , argDoc =
--               Documentation
--                 { rawDocText = Just "a copy `GskRenderNode`"
--                 , sinceVersion = Nothing
--                 }
--           , argScope = ScopeTypeInvalid
--           , argClosure = -1
--           , argDestroy = -1
--           , argCallerAllocates = False
--           , argCallbackUserData = False
--           , transfer = TransferNothing
--           }
--       ]
-- Lengths: []
-- returnType: Just (TInterface Name { namespace = "Gsk" , name = "RenderNode" })
-- throws : False
-- Skip return : False

foreign import ccall "gsk_copy_node_get_child" gsk_copy_node_get_child :: 
    Ptr CopyNode ->                         -- node : TInterface (Name {namespace = "Gsk", name = "CopyNode"})
    IO (Ptr Gsk.RenderNode.RenderNode)

-- | Gets the child node that is getting drawn by the given /@node@/.
-- 
-- /Since: 4.22/
copyNodeGetChild ::
    (B.CallStack.HasCallStack, MonadIO m, IsCopyNode a) =>
    a
    -- ^ /@node@/: a copy @GskRenderNode@
    -> m Gsk.RenderNode.RenderNode
    -- ^ __Returns:__ the child @GskRenderNode@
copyNodeGetChild node = liftIO $ do
    node' <- unsafeManagedPtrCastPtr node
    result <- gsk_copy_node_get_child node'
    checkUnexpectedReturnNULL "copyNodeGetChild" result
    result' <- (newPtr Gsk.RenderNode.RenderNode) result
    touchManagedPtr node
    return result'

#if defined(ENABLE_OVERLOADING)
data CopyNodeGetChildMethodInfo
instance (signature ~ (m Gsk.RenderNode.RenderNode), MonadIO m, IsCopyNode a) => O.OverloadedMethod CopyNodeGetChildMethodInfo a signature where
    overloadedMethod = copyNodeGetChild

instance O.OverloadedMethodInfo CopyNodeGetChildMethodInfo a where
    overloadedMethodInfo = P.Just (O.ResolvedSymbolInfo {
        O.resolvedSymbolName = "GI.Gsk.Objects.CopyNode.copyNodeGetChild",
        O.resolvedSymbolURL = "https://hackage.haskell.org/package/gi-gsk-4.0.8/docs/GI-Gsk-Objects-CopyNode.html#v:copyNodeGetChild"
        })


#endif


